"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _isEqual2 = _interopRequireDefault(require("lodash/isEqual"));

var _react = require("react");

var _delay = _interopRequireDefault(require("delay"));

var _promise = require("./promise");

var _selectObjectValue = _interopRequireDefault(require("./util/select-object-value"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const reducer = (state, action) => {
  let newState;

  switch (action.type) {
    case 'reset':
      newState = {
        status: 'none',
        value: undefined,
        withinThreshold: undefined
      };
      break;

    case 'pending':
      {
        newState = {
          status: 'pending',
          value: undefined,
          withinThreshold: action.payload
        };
        break;
      }

    case 'rejected':
      newState = { ...state,
        status: 'rejected',
        value: action.payload
      };
      break;

    case 'fulfilled':
      newState = { ...state,
        status: 'fulfilled',
        value: action.payload
      };
      break;

    /* istanbul ignore next */

    default:
      throw new Error('Unknown action type');
  } // Keep the same state reference if they are strictly equal to avoid rerenders


  return (0, _isEqual2.default)(newState, state) ? state : newState;
};

const getInitialState = (promise, options) => ({
  status: !promise ? 'none' : 'pending',
  value: undefined,
  withinThreshold: !promise ? undefined : options.thresholdMs > 0
});

const useResolvePromise = (promise, dispatch, options) => {
  const {
    thresholdMs
  } = options;
  (0, _react.useEffect)(() => {
    // Short-circuit if there's no promise
    if (!promise) {
      dispatch({
        type: 'reset'
      });
      return;
    }

    let cancelDelay; // If there's no delay, set status to 'pending' right away
    // Otherwise wait for the specified elay

    if (thresholdMs > 0) {
      dispatch({
        type: 'pending',
        payload: true
      });
      cancelDelay = (0, _promise.resolvePromise)((0, _delay.default)(thresholdMs), {
        then: () => dispatch({
          type: 'pending',
          payload: false
        })
      });
    } else {
      dispatch({
        type: 'pending',
        payload: false
      });
    }

    const cancelResolve = (0, _promise.resolvePromise)(promise, {
      then: value => dispatch({
        type: 'fulfilled',
        payload: value
      }),
      catch: err => dispatch({
        type: 'rejected',
        payload: err
      }),
      finally: () => {
        cancelDelay && cancelDelay();
      }
    }, true);
    return () => {
      // Cancel delay & cancel resolving promise
      cancelResolve();
      cancelDelay && cancelDelay();
    };
  }, [promise]);
};

const useNotifySettled = (state, options) => {
  const {
    onSettle,
    onSettleDelayMs: onSettleDelayMs_
  } = options;
  const onSettleDelayMs = (0, _react.useMemo)(() => {
    if (typeof onSettleDelayMs_ === 'number') {
      return {
        fulfilled: onSettleDelayMs_,
        rejected: onSettleDelayMs_
      };
    }

    return {
      fulfilled: 0,
      rejected: 0,
      ...onSettleDelayMs_
    };
  }, [onSettleDelayMs_]);
  const stateRef = (0, _react.useRef)();
  (0, _react.useEffect)(() => {
    // Short-circuit if there's no callback
    if (!onSettle) {
      return;
    } // Short-circuit if state hasn't changed


    if (stateRef.current === state) {
      return;
    }

    stateRef.current = state; // Skip if the promise is not settled

    if (state.status !== 'fulfilled' && state.status !== 'rejected') {
      return;
    }

    const timeoutDelay = (0, _selectObjectValue.default)(onSettleDelayMs, state, 0);

    if (timeoutDelay > 0) {
      const timeoutId = setTimeout(() => onSettle(state), timeoutDelay);
      return () => clearTimeout(timeoutId);
    }

    onSettle(state);
  }, [state, onSettle, onSettleDelayMs]);
};

const usePromiseState = (promise, options) => {
  options = (0, _react.useMemo)(() => ({
    thresholdMs: 0,
    onSettleDelayMs: 0,
    onSettle: undefined,
    statusMap: undefined,
    ...options
  }), [options]);
  const initialState = (0, _react.useMemo)(() => getInitialState(promise, options));
  const [state, dispatch] = (0, _react.useReducer)(reducer, initialState); // Observe promise and update the state accordingly, respecting `thresholdMs`

  useResolvePromise(promise, dispatch, options); // Call `onSettled` whenever the promise settles, respecting `onSettleDelayMs`

  useNotifySettled(state, options); // Return the state, transforming it according to `statusMap`

  return (0, _react.useMemo)(() => ({ ...state,
    status: (0, _selectObjectValue.default)(options.statusMap, state, state.status)
  }), [state, options.statusMap]);
};

var _default = usePromiseState;
exports.default = _default;
module.exports = exports.default;